/**
 * apeman task to generate captcha images.
 * @memberof module:apeman-task-captcha/lib
 * @function define
 * @param {string} filename - Data file name.
 * @param {object} [options] - Optional settings.
 * @param {number} [options.count=25] - Count of images
 * @param {string} [options.imageDir='captcha'] - Images directory.
 * @param {boolean} [options.clean] - Cleanup image directory before generating.
 * @returns {function} - Defined task function.
 */

'use strict'

const path = require('path')
const mkdirp = require('mkdirp')
const writeout = require('writeout')
const randomval = require('randomval')
const rimraf = require('rimraf')
const captchaSVG = require('./captcha/captcha_s_v_g')
const uuid = require('uuid')
const co = require('co')

/** @lends define */
function define (filename, options = {}) {
  let imageDir = options.imageDir || 'captcha'
  let clean = !!options.clean
  let count = Number(options.count || 25)

  let images = define.imagesData(count)

  function _relativePath (filename) {
    return path.relative(process.cwd(), filename)
  }

  /**
   * Defined task.
   * @function task
   * @param {object} context - Apeman task context.
   * @returns {Promise}
   */
  function task (context) {
    let { logger } = context
    return co(function * () {
      if (clean) {
        yield new Promise((resolve, reject) =>
          rimraf(imageDir, (err) => err ? reject(err) : resolve())
        )
      }
      yield new Promise((resolve, reject) =>
        mkdirp(imageDir, (err) => err ? reject(err) : resolve())
      )
      for (let id of Object.keys(images)) {
        // TODO Convert to png
        let filename = path.join(imageDir, id + '.svg')
        let svg = yield captchaSVG(images[ id ])
        yield writeout(filename, svg, {
          mkdirp: true
        })
        logger.trace(_relativePath(filename))
      }
      let data = {}
      Object.keys(images).forEach(id => {
        let filename = path.join(imageDir, id + '.svg')
        data[ filename ] = images[ id ];
      })
      yield writeout(filename, JSON.stringify(data, null, 2))
      logger.debug(`File generated: ${_relativePath(filename)}`)
    })
  }

  // Description of this task.
  task.$desc = "Generate captcha images"
  return task;
}

define.imagesData = function images (count) {
  let images = {}
  for (let i = 0; i < count; i++) {
    let id = uuid.v4()
    images[ id ] = String(randomval.randomInt(1000, 9999))
  }
  return images
}

module.exports = define
